/**
 * 
 */
package snippet.gef.sample2;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ZoomInAction;
import org.eclipse.gef.ui.actions.ZoomOutAction;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.GraphicalEditor;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;

import snippet.gef.sample2.gef.part.factory.ModelDesignPartFactory;
import snippet.gef.sample2.model.Model;

/**
 * @author amergey
 * 
 */
public class ModelDesignGraphEditor extends GraphicalEditor {
	
	private boolean mDirty;
	private IContentOutlinePage mOutlinePage = null;

	public ModelDesignGraphEditor() {
		setEditDomain(new DefaultEditDomain(this));
	}

	/**
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor#isDirty()
	 */
	@Override
	public boolean isDirty() {
		return mDirty;
	}

	public void setDirty(boolean pDirty) {
		mDirty = pDirty;
		firePropertyChange(PROP_DIRTY);
	}

	/**
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor#configureGraphicalViewer()
	 */
	@Override
	protected void configureGraphicalViewer() {
		super.configureGraphicalViewer();
		
		GraphicalViewer viewer = getGraphicalViewer();
		viewer.setEditPartFactory(new ModelDesignPartFactory(this));

		// zoom
		ScalableRootEditPart rootEditPart = new ScalableRootEditPart();
		viewer.setRootEditPart(rootEditPart);

		ZoomManager manager = rootEditPart.getZoomManager();
		getActionRegistry().registerAction(new ZoomInAction(manager));
		getActionRegistry().registerAction(new ZoomOutAction(manager));

		double[] zoomLevels = new double[] { 0.25, 0.5, 0.75, 1.0, 1.5, 2.0,
				2.5, 3.0, 4.0, 5.0, 10.0, 20.0 };
		manager.setZoomLevels(zoomLevels);

		List<String> zoomContributions = new ArrayList<String>();
		zoomContributions.add(ZoomManager.FIT_ALL);
		zoomContributions.add(ZoomManager.FIT_HEIGHT);
		zoomContributions.add(ZoomManager.FIT_WIDTH);
		manager.setZoomLevelContributions(zoomContributions);
	}

	@Override
	protected void initializeGraphicalViewer() {

		Model model = new Model(3);
		GraphicalViewer viewer = getGraphicalViewer();
		viewer.setContents(model);
	}

	@Override
	public void doSave(IProgressMonitor pMonitor) {
		setDirty(false);
	}

	@Override
	public Object getAdapter(Class type) {
		if (type.equals(ZoomManager.class))
			return ((ScalableRootEditPart) getGraphicalViewer()
					.getRootEditPart()).getZoomManager();
		else if (type.equals(IContentOutlinePage.class)) {
			if (mOutlinePage == null) {
				mOutlinePage = new ThumbailOutlineView();
			}
			return mOutlinePage;
		} else {
			return super.getAdapter(type);
		}
	}

	/**
	 * @author amergey
	 * 
	 */
	private class ThumbailOutlineView extends ContentOutlinePage {
		private SashForm mSash;
		private ScrollableThumbnail mThumbnail;
		private DisposeListener mDisposeListener;

		public ThumbailOutlineView() {
			super(new TreeViewer());
		}

		@Override
		public void init(IPageSite pageSite) {
			super.init(pageSite);
		}

		@Override
		public void createControl(Composite parent) {
			mSash = new SashForm(parent, SWT.VERTICAL);
			Canvas canvas = new Canvas(mSash, SWT.BORDER);

			LightweightSystem lws = new LightweightSystem(canvas);

			mThumbnail = new ScrollableThumbnail(
					(Viewport) ((AbstractGraphicalEditPart) getGraphicalViewer()
							.getRootEditPart()).getFigure());

			mThumbnail.setSource(((ScalableRootEditPart) getGraphicalViewer()
					.getRootEditPart())
					.getLayer(LayerConstants.PRINTABLE_LAYERS));

			lws.setContents(mThumbnail);

			mDisposeListener = new DisposeListener() {

				@Override
				public void widgetDisposed(DisposeEvent e) {
					if (mThumbnail != null) {
						mThumbnail.deactivate();
						mThumbnail = null;
					}

				}
			};

			getGraphicalViewer().getControl().addDisposeListener(
					mDisposeListener);
		}

		@Override
		public Control getControl() {
			return mSash;
		}

		@Override
		public void dispose() {
			getSelectionSynchronizer().removeViewer(getViewer());

			if (getGraphicalViewer().getControl() != null
					&& !getGraphicalViewer().getControl().isDisposed()) {
				getGraphicalViewer().getControl().removeDisposeListener(
						mDisposeListener);
			}

			super.dispose();
		}
	}
}
