/**
 * 
 */
package snippet.gef.sample2.gef.part;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.viewers.TextCellEditor;

import snippet.gef.sample2.gef.editor.EntityEditManager;
import snippet.gef.sample2.gef.editor.LabelCellEditorLocator;
import snippet.gef.sample2.gef.figure.EntityFigure;
import snippet.gef.sample2.gef.policy.EntityDirectEditPolicy;
import snippet.gef.sample2.model.Attribute;
import snippet.gef.sample2.model.Entity;

/**
 * @author amergey
 * 
 */
public class EntityPart extends AbstractGraphicalEditPart implements PropertyChangeListener{
	@Override
	public void activate() {
		if (isActive())
			return;
		super.activate();
		((Entity)getModel()).addPropertyChangeListener(this);
	}

	@Override
	public void deactivate() {
		super.deactivate();
		((Entity)getModel()).removePropertyChangeListener(this);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	@Override
	protected IFigure createFigure() {
		return new EntityFigure();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new EntityDirectEditPolicy());
	}

	@Override
	protected void refreshVisuals() {
		EntityFigure figure = (EntityFigure) getFigure();
		Entity model = (Entity) getModel();

		figure.setEntityLabel(model.getLabel());
	}
	
	@Override
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT){
			performDirectEdit();
		}
	}
	
	private void performDirectEdit() {
		new EntityEditManager(this,TextCellEditor.class, new LabelCellEditorLocator(
				(EntityFigure) getFigure())).show();
	}
	
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@Override
	protected List getModelChildren() {
		return new ArrayList<Attribute>(((Entity)getModel()).getAllAttributes());
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		if(evt.getPropertyName().equals("label")){
			refreshVisuals();
		}
	}
}
